# Set the variables to start.
endtoend=0;
stepper=0;
installsite=0;
testname="";

# Ensure we are not using the recorder.
cp tests/acceptance_norecorder.suite.yml tests/acceptance.suite.yml

# Get the options from the command line.
while getopts 'rsiet:' OPTION; do
  case "$OPTION" in
    r)
      cp tests/acceptance_recorder.suite.yml tests/acceptance.suite.yml
      ;;
    s)
      stepper=1
      ;;
    i)
      installsite=1
      ;;
    e)
      endtoend=1
      ;;
    t)
      testname=$OPTARG
      ;;
    ?)
      echo "script usage: [-rsiet]" >&2
      exit 1
      ;;
  esac
done

# Ensure that we have either end to end or specific test.
if [[ $endtoend == 0 ]] && [[ $testname == "" ]]; then
  echo "You must specify a test if not using end to end tests";
  exit 1;
fi;

# Remove all the old test data.
rm -rf tests/_output
rm -rf tests/_data

if [[ $installsite == 1 ]]; then
  # Generate a file name for the sql file.
  sqlfile=$(cat /proc/sys/kernel/random/uuid)
  sqlfile="$sqlfile.sql"

  # Install a site and get an sql file from site install.
  echo "*********************************************************"
  echo "Installing site ..."
  drush si uw_base_profile --db-url=mysql://drupal:drupal@mariadb/drupal --account-name=wcmsadmin --account-pass=a --site-name="WCMS Test" -y
  drush sql-dump > "$sqlfile"
  echo "Done installing site."
  echo "*********************************************************"
fi

if [[ $endtoend == 1 ]]; then

  # Directory to iterate through
  directory="tests/acceptance"

  # Get the file names of the tests.
  files=()
  for file in "$directory"/*; do
      if [[ -f "$file" ]]; then
          file="${file//tests\/acceptance\//}"
          file="${file//\.php/}"
          files+=("$file")
      fi
  done
else
  files=("$testname")
fi

# Perform the tests using the files.
for value in "${files[@]}"; do
  echo "*********************************************************"
  echo "Current Test: $value"
  echo "*********************************************************"

  if [[ $endtoend == 1 ]]; then
    echo "Stopping chrome driver ..."
    echo "*********************************************************"
    pgrep chrome | xargs kill -9
  fi

  if [[ $installsite == 1 ]]; then
    echo "Dropping database ..."
    echo "*********************************************************"
    drush sql-drop -y
    echo "Importing database ..."
    echo "*********************************************************"
    drush sql-cli < "$sqlfile"
  fi

  if [[ $endtoend == 1 ]]; then
    echo "Starting chrome driver ..."
    echo "*********************************************************"
    chromedriver --port=9515 --url-base=/wd/hub &
    echo "Sleeping ..."
    echo "*********************************************************"
    sleep 15s
    echo ""
  fi

  echo "*********************************************************"
  echo "Starting test ... $value"
  echo "*********************************************************"
  if [[ $stepper == 1 ]]; then
    php vendor/bin/codecept run acceptance "$value" --fail-fast --steps
  else
    php vendor/bin/codecept run acceptance "$value" --fail-fast
  fi
  if [[ $? -ne 0 ]]; then
    if [[ $installsite == 1 ]]; then
      rm "$sqlfile"
    fi
    echo ""
    echo "*********************************************************"
    echo "Test $value failed, done test."
    echo "*********************************************************"
    echo ""
    break
  fi
  echo ""
  echo "*********************************************************"
  echo "Done test $value."
  echo "*********************************************************"
  echo ""
done
